/****************************************************************************

    sdl.c
    SDL interface

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/

#include "doubledash.h"
#include "player.h"
#include "sdl.h"
#include "utils.h"
#include <SDL.h>
#include <SDL_keysym.h>
#include <SDL_image.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>


/* main window */
SDL_Surface *sdl_main_display;



/* special keys */
int sdl_key_esc;
int sdl_key_space; /* pause */
int sdl_last_key;



/*
  Initialize SDL
 */

void sdl_init()
{

  atexit( sdl_close );

  if((SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)==-1))
    { 
      printf("Could not initialize SDL: %s.\n", SDL_GetError());
      exit(-1);
  }

  /* Initialize the display in a 16-bit palettized mode */

  sdl_main_display = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_HWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF);

  if ( sdl_main_display == NULL ) 
    {
      fprintf(stderr, "Couldn't set video mode: %s\n",
	      SDL_GetError());
      exit(1);
    }

  SDL_ShowCursor( SDL_DISABLE );
  SDL_EnableKeyRepeat( SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL );

  player_set_keys( 0, SDLK_UP, SDLK_RIGHT, SDLK_DOWN, SDLK_LEFT, SDLK_RCTRL );
  player_set_keys( 1, SDLK_w, SDLK_d, SDLK_s, SDLK_a, SDLK_TAB );  
  player_set_keys( 2, SDLK_i, SDLK_l, SDLK_k, SDLK_j, SDLK_n );  
  player_set_keys( 3, SDLK_KP8, SDLK_KP6, SDLK_KP5, SDLK_KP4, SDLK_KP0 );  
  sdl_reset_keyboard();

  srand( time(NULL) );

}



/*
  Quit SDL
 */

void sdl_close( void )
{
  /* Shutdown all subsystems */
  SDL_Quit();
}




/*
  polls all pending events
*/

void sdl_poll_events( void )
{
  SDL_Event event;

  while( SDL_PollEvent( &event ) )
    {
      switch( event.type )
	{
	case SDL_KEYDOWN:

	  sdl_last_key = event.key.keysym.sym;

	  player_keypress( event.key.keysym.sym );

	  switch( event.key.keysym.sym )
	    {
	    case SDLK_ESCAPE:
	      sdl_key_esc=1;
	      break;

	    case SDLK_SPACE:
	      sdl_key_space=1;
	      break;

	    default:
	      break;
	    }
	  
	  break;

	      
	case SDL_KEYUP:

	  player_keydepress( event.key.keysym.sym );

	  switch( event.key.keysym.sym )
	    {
	    case SDLK_ESCAPE:
	      /* esc can be only reseted */
	      break;

	    case SDLK_SPACE:
	      /* space can be only reseted */
	      break;

	    default:
	      break;
	    }
	  
	  break;
	  
	  /* SDL_QUIT event (window close) */
	case SDL_QUIT:
	  exit(0);
	  break;

	default:
	  break;
	}
      
    }

}





/*
  resets keyboard status
*/

void sdl_reset_keyboard( void )
{
  sdl_poll_events();

  sdl_key_esc=0;
  sdl_key_space=0;
  sdl_last_key=0;

}



/* 
   wait for keypress
*/
int sdl_get_key( void )
{
  sdl_reset_keyboard();
  while( !sdl_last_key )
    {
      SDL_Delay( 100 );
      sdl_poll_events();
    }
  return sdl_last_key;
}



/*
  return esc key state
 */
int sdl_esc_pressed( void )
{
  return sdl_key_esc;
}


/*
  return space key state
 */
int sdl_space_pressed( void )
{
  return sdl_key_space;
}




/*
  Create a 32-bit surface with the bytes of each pixel in R,G,B,A
  Taken from /usr/share/doc/SDL-devel-1.2.5/html/sdlcreatergbsurface.html
*/
SDL_Surface *sdl_new_surface( int width, int height )
{
  
  SDL_Surface *surface;
  SDL_Surface *result;
  Uint32 rmask, gmask, bmask, amask;

  /* SDL interprets each pixel as a 32-bit number, so our masks must depend
     on the endianness (byte order) of the machine */
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
  rmask = 0xff000000;
  gmask = 0x00ff0000;
  bmask = 0x0000ff00;
  amask = 0x000000ff;
#else
  rmask = 0x000000ff;
  gmask = 0x0000ff00;
  bmask = 0x00ff0000;
  amask = 0xff000000;
#endif
    
  surface = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 32,
				 rmask, gmask, bmask, amask);
  if(surface == NULL) {
    fprintf(stderr, "CreateRGBSurface failed: %s\n", SDL_GetError());
    exit(1);
  }

  result = SDL_ConvertSurface( surface, sdl_main_display->format, 
                                          sdl_main_display->flags );
  SDL_FreeSurface( surface );
  return result;

}




/*
  load sprite image
*/
Sdl_Sprite *sdl_sprite_load( char *name, int columns, int rows )
{
  int i, j, width, height;
  char *fullname=utils_fullname( name );
  SDL_Surface *image;
  SDL_Rect src, dst;
  Sdl_Sprite *result;

  if( rows<=0 || columns<=0 )
    {
      fprintf( stderr, "DoubleDash: Wrong sdl_load_sprite parameters\n" );
      exit(1);
    }


  image=IMG_Load( fullname );
  
  if( image==NULL )
    {
      fprintf( stderr, "DoubleDash: No such image %s\n", fullname );
      exit(1);
    }

  width=image->w/columns;
  height=image->h/rows;

  result = utils_malloc( sizeof(Sdl_Sprite) );
  result->images_num=rows*columns;
  result->image = utils_malloc( result->images_num*sizeof(SDL_Surface*) );

  src.w=dst.w=width;
  src.h=dst.h=height;
  dst.x=dst.y=0;

  for( j=0; j<rows; ++j )
    for( i=0; i<columns; ++i )
      {
	src.x=i*width;
	src.y=j*height;
	result->image[i+j*columns]=sdl_new_surface( width, height );
	if( SDL_BlitSurface( image, &src, result->image[i+j*columns], &dst ) )
	  {
	    fprintf( stderr, "DoubleDash: Error blitting %s\n", fullname );
	    exit(1);
	  }
      }


  SDL_FreeSurface( image );

  return result;

}




/*
  free sprite image
*/
void sdl_sprite_free( Sdl_Sprite *sprite )
{
  int i;
  for( i=0; i<sprite->images_num; ++i )
    SDL_FreeSurface( sprite->image[i] );

  free( sprite->image );
  free( sprite );
}


/*
  draw sprite
 */
void sdl_sprite_draw( Sdl_Sprite *sprite, int x, int y, int frame )
{
  SDL_Rect src, dst;
  SDL_Surface *image=sprite->image[frame%sprite->images_num];

  src.w=dst.w=image->w;
  src.h=dst.h=image->h;
  src.x=src.y=0;
  dst.x=x;
  dst.y=y;

  if( SDL_BlitSurface( image, &src, sdl_main_display, &dst ) )
    {
      fprintf( stderr, "DoubleDash: Error displaying sprite.\n" );
      exit(1);
    }
}


/*
  flip double buffer
 */
void sdl_flip( void )
{
  SDL_Flip(sdl_main_display);
}
 


/*
  set clip rectangle
 */ 
void sdl_clip( int x, int y, int w, int h )
{
  SDL_Rect rect;
  rect.x=x;
  rect.y=y;
  rect.w=w;
  rect.h=h;
  SDL_SetClipRect( sdl_main_display, &rect );
}


/*
  resetset clip rectangle
 */ 
void sdl_unclip( void )
{
  SDL_SetClipRect( sdl_main_display, NULL );
}
